'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var inquirer = _interopDefault(require('inquirer'));
var chalk = _interopDefault(require('chalk'));
var path = _interopDefault(require('path'));
var fs = _interopDefault(require('fs'));
var childProcess = _interopDefault(require('child_process'));
var ncu = _interopDefault(require('npm-check-updates'));

var exec$1 = childProcess.exec;
// ----------------------------------------------------------------
var installModule = (function (module, depType, name, cb) {
  fs.stat(path.resolve("./node_modules/" + module), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow(name + " not detected..."));
      inquirer.prompt([{
        type: "confirm",
        name: "install",
        message: "Install " + name + " now?",
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          console.log(chalk.blue("Installing " + name + " via npm..."));
          exec$1("npm i " + depType + " " + module, function (error, stdout, stderr) {
            console.log(chalk.green("Added " + name));
            if (error !== null) {
              console.log("exec error: " + error);
            }
            cb();
          });
        } else {
          console.log(chalk.yellow(name + " was not installed."));
        }
      });
    } else {
      console.log(chalk.green(name + " already exists."));
      cb();
    }
  });
})

// ----------------------------------------------------------------
var addTape = (function (callback) {
  installModule("tape", "-D", "Tape", function () {
    callback();
  });
})

var exec = childProcess.exec;
// ----------------------------------------------------------------
var runTests = (function (dir, entry) {
  addTape(function () {
    fs.stat(path.resolve("./node_modules/babel-preset-es2015"), function (err, stats) {
      if (err && err.code !== "ENOENT") console.log(err);
      if (err && err.code === "ENOENT") {
        console.log(chalk.yellow("Babel is not installed locally. \n(To use ES6 with your tests, please install Babel with the Babel preset task.)"));
        console.log(chalk.yellow("----------------"));
        console.log(chalk.yellow("Running project tests with ES5..."));
      } else {
        console.log(chalk.yellow("Running project tests with ES6..."));
      }
    });
    exec(path.resolve(__dirname, "../node_modules/.bin/nyc") + " --require " + path.resolve(__dirname, "../node_modules/babel-register") + " --reporter=lcov --reporter=text " + path.resolve(__dirname, "../node_modules/.bin/tape") + " '" + path.resolve(dir) + "/**/*.js' | " + path.resolve(__dirname, "../node_modules/.bin/faucet"), function (error, stdout, stderr) {
      if (error !== null) {
        console.log("exec error: " + error);
      }
      console.log(stdout);
      console.log(stderr);
    });
  });
})

var exec$2 = childProcess.exec;
// ----------------------------------------------------------------
var rollupFiles = (function (file, output) {
  console.log("Rolling files into bundle...");
  exec$2("BABEL_ENV=build " + path.resolve(__dirname, "../node_modules/.bin/rollup") + " --config " + path.resolve(__dirname, "../vendor/rollup.config.js") + " " + path.resolve(file) + " -o " + path.resolve(output), function (error, stdout, stderr) {
    if (error !== null) {
      console.log("exec error: " + error);
    }
    console.log("Rolled " + file + " into bundle at " + output + ".");
  });
})
// --config ../vendor/rollup.config.js

// ----------------------------------------------------------------
var addBabelPreset = (function () {

  installModule("babel-preset-es2015", "-D", "Babel Preset", function () {
    installModule("babel-preset-es2015-rollup", "-D", "Babel Preset for Rollup", function () {
      addBabelRc();
    });
  });

  // --------------------------------

  var addBabelRc = function addBabelRc() {
    fs.stat(path.resolve(".babelrc"), function (err, stats) {
      if (err && err.code !== "ENOENT") console.log(err);
      if (err && err.code === "ENOENT") {
        console.log(chalk.yellow("No .babelrc file detected..."));
        console.log(chalk.blue("Creating .babelrc file..."));
        var oldFile = fs.createReadStream(path.resolve(__dirname, "../vendor/.babelrc"));
        var newFile = fs.createWriteStream(path.resolve(".babelrc"));
        oldFile.pipe(newFile);
        console.log(chalk.green(".babelrc file created."));
      } else {
        console.log(chalk.green(".babelrc already exists."));
      }
    });
  };
})

// ----------------------------------------------------------------
var addWallabyConfig = (function () {
  console.log(__dirname);
  fs.stat(path.resolve("wallaby.js"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No wallaby.js file detected..."));
      console.log(chalk.blue("Creating wallaby.js file..."));
      var oldFile = fs.createReadStream(path.resolve(__dirname, "../vendor/wallaby.js"));
      var newFile = fs.createWriteStream(path.resolve("wallaby.js"));
      oldFile.pipe(newFile);
      console.log(chalk.green("wallaby.js file created."));
    } else {
      console.log(chalk.green("wallaby.js already exists."));
    }
  });
})

// ----------------------------------------------------------------
var addEzyConfig = (function () {

  fs.stat(path.resolve("ezy.config.json"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No ezy.config.json file detected..."));
      console.log(chalk.blue("Creating ezy.config.json file..."));
      var oldFile = fs.createReadStream(path.resolve(__dirname, "../vendor/ezy.config.json"));
      var newFile = fs.createWriteStream(path.resolve("ezy.config.json"));
      oldFile.pipe(newFile);
      console.log(chalk.green("ezy.config.json file created."));
    } else {
      console.log(chalk.green("ezy.config.json already exists."));
    }
  });
})

// ----------------------------------------------------------------
var addESLint = (function () {

  fs.stat(path.resolve(".eslintrc"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No .eslintrc file detected..."));
      console.log(chalk.blue("Creating .eslintrc file..."));

      // fs.writeFile(path.resolve(".babelrc"), "{\n  \"presets\": [\"es2015\"]\n}", (err) => {
      //   if (err) throw err;
      //   console.log(chalk.green("Added preset to .babelrc file."));
      // });
      var oldFile = fs.createReadStream(path.resolve(__dirname, "../vendor/.eslintrc"));
      var newFile = fs.createWriteStream(path.resolve(".eslintrc"));
      oldFile.pipe(newFile);
      console.log(chalk.green(".eslintrc file created."));
    } else {
      console.log(chalk.green(".eslintrc already exists."));
    }
  });
})

var exec$7 = childProcess.exec;
// ----------------------------------------------------------------
var viewCoverage = (function () {
  console.log(chalk.yellow("Opening coverage report in the browser..."));
  exec$7("open " + path.resolve("coverage/lcov-report/index.html"), function (error, stdout, stderr) {
    if (error !== null) {
      console.log("exec error: " + error);
    }
  });
})

// ----------------------------------------------------------------
var addBrowserAssets = (function (dir, file, callback) {
  fs.stat(path.resolve(dir + "/index.html"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("index.html not detected at " + path.normalize(dir + "/index.html") + "..."));
      inquirer.prompt([{
        type: "confirm",
        name: "install",
        message: "Create index.html now?",
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          (function () {
            console.log(chalk.blue("Creating index.html..."));
            var oldFile = fs.createReadStream(path.resolve(__dirname, "../vendor/index.html"));
            var newFile = fs.createWriteStream(path.resolve(dir + "/index.html"));

            newFile.on("pipe", function (src) {
              console.error("Writing now...");
              if (src === oldFile) {
                fs.readFile(path.resolve(dir + "/index.html"), function (err, data) {
                  if (err) console.log(err);
                  data = data.toString().split("\n");
                  data.splice(5, 1, "  <script src=\"./" + file + "\"></script>");
                  var text = data.join("\n");
                  fs.writeFile(path.resolve(dir + "/index.html"), text, function (err, stdout) {
                    if (err) console.log(err);
                    console.log(chalk.green("index.html file created."));
                    callback();
                  });
                });
              }
            });
            oldFile.pipe(newFile);
          })();
        } else {
          console.log(chalk.red("An index.html file is required to run projects in the browser."));
        }
      });
    } else {
      console.log(chalk.green("index.html exists."));
      callback();
    }
  });
})

var exec$8 = childProcess.exec;
// ----------------------------------------------------------------
var runProject = {
  node: function node(file) {
    console.log(chalk.yellow("Running " + file + " with Node..."));
    exec$8("node " + path.resolve(file), function (error, stdout, stderr) {
      if (error !== null) {
        console.log("exec error: " + error);
      }
      console.log(stdout);
      console.log(stderr);
    });
  },
  es6: function es6(file) {
    console.log(chalk.yellow("Running " + file + " with ES6..."));
    exec$8("BABEL_ENV=build " + path.resolve(__dirname, "../node_modules/.bin/rollup") + " --config " + path.resolve(__dirname, "../vendor/rollup.config.js") + " " + path.resolve(file) + " | node ", function (error, stdout, stderr) {
      if (error !== null) {
        console.log("exec error: " + error);
      }
      console.log(stdout);
      console.log(stderr);
    });
  },
  browser: function browser(dir, file) {
    addBrowserAssets(dir, file, function () {
      console.log(chalk.yellow("Running " + dir + " with BrowserSync..."));
      exec$8(path.resolve(__dirname, "../node_modules/.bin/browser-sync") + " start --server " + path.resolve(dir) + " --files " + path.resolve(dir), function (error, stdout, stderr) {
        if (error !== null) {
          console.log("exec error: " + error);
        }
        console.log(stdout);
        console.log(stderr);
      });
    });
  }
};

// ----------------------------------------------------------------
var updateDependencies = {
  check: function check() {
    console.log(chalk.yellow("Checking for new versions of your dependencies..."));
    ncu.run({
      packageFile: path.resolve("package.json"),
      jsonUpgraded: true,
      upgrade: false
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green("Everything is up to date. :)"));
      } else {
        console.log(toUpgrade);
      }
    });
  },
  run: function run() {
    console.log(chalk.yellow("Updating your dependencies..."));
    ncu.run({
      packageFile: path.resolve("package.json"),
      jsonUpgraded: true,
      upgrade: true
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green("Everything is up to date. :)"));
      } else {
        console.log(toUpgrade);
      }
    });
  }

};

// ----------------------------------------------------------------

var questions = [{
  type: "list",
  name: "inputValue",
  message: "What would you like to do?",
  choices: ["1. Run Project", new inquirer.Separator(), "2. Test Project", "3. View Coverage", new inquirer.Separator(), "4. Bundle Project", new inquirer.Separator(), "5. Check Updates", "6. Run Updates", "7. Add Babel Preset", "8. Add Wallaby Config", "9. Add ESLint", "10. Add ezy Config", new inquirer.Separator()]
}];

var taskList = (function (configure) {
  var config = require(configure); // Weird Require.
  var entry, entryDir, outputDir, output, testDir;
  if (config.entry) {
    entry = config.entry;
  } else {
    entry = "src/index.js";
  }
  if (config.entryDir) {
    entryDir = config.entryDir;
  } else {
    entryDir = "src";
  }
  if (config.outputDir) {
    outputDir = config.outputDir;
  } else {
    outputDir = "dist";
  }
  if (config.output) {
    output = config.output;
  } else {
    output = "bundle.js";
  }
  if (config.testDir) {
    testDir = config.testDir;
  } else {
    testDir = "tests";
  }
  // --------------------------------
  inquirer.prompt(questions).then(function (answers) {
    var a = answers.inputValue;

    if (a === "1. Run Project") {
      inquirer.prompt([{
        type: "list",
        name: "runWhere",
        message: "Where would you like to run the project?",
        choices: ["Node", "ES6", "Browser"]
      }]).then(function (answers) {
        if (answers.runWhere === "Node") {
          runProject.node(outputDir + "/" + output);
        } else if (answers.runWhere === "Browser") {
          runProject.browser(outputDir, output);
        } else if (answers.runWhere === "ES6") {
          runProject.es6(entryDir + "/" + entry);
        }
      });
    } else if (a === "2. Test Project") {
      runTests(testDir, entryDir);
    } else if (a === "3. View Coverage") {
      viewCoverage();
    } else if (a === "4. Bundle Project") {
      rollupFiles(entryDir + "/" + entry, outputDir + "/" + output);
    } else if (a === "5. Check Updates") {
      updateDependencies.check();
    } else if (a === "6. Run Updates") {
      updateDependencies.run();
    } else if (a === "7. Add Babel Preset") {
      addBabelPreset();
    } else if (a === "8. Add Wallaby Config") {
      addWallabyConfig();
    } else if (a === "9. Add ESLint") {
      addESLint();
    } else if (a === "10. Add ezy Config") {
      addEzyConfig();
    }
  });
})

// ----------------------------------------------------------------
var bin = function bin() {
  // TODO: Check for ezy.config.json, chck for ezy section in package json,
  //       else suggest config file is made.
  fs.stat(path.resolve("ezy.config.json"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.red("No ezy.config.json file has been found."));
      inquirer.prompt([{
        type: "confirm",
        name: "install",
        message: "Create an ezy.config.json file now?",
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          addEzyConfig();
        } else {
          console.log("Please create a config file to use ezy.");
        }
      });
    } else {
      taskList(path.resolve("ezy.config.json"));
    }
  });
};

exports.bin = bin;